package yt_downloader

import (
	"crypto/sha256"
	"downloader/internal/core/ports/downloader"
	"downloader/pkg/files"
	"fmt"
	"go.uber.org/zap"
	"io"
	"io/fs"
	"log"
	"os"
	"os/exec"
	"regexp"
	"strings"
	"time"
)

type YtDownloader struct {
	outputDirectory  string
	tempDirectory    string
	checkFrequencyMs time.Duration
	logger           *zap.SugaredLogger
}

func New(logger *zap.SugaredLogger) downloader.Downloader {
	return &YtDownloader{
		outputDirectory:  "/home/hermansen/Downloads/yt",
		tempDirectory:    "/tmp/downloader",
		checkFrequencyMs: 5000,
		logger:           logger,
	}
}

func init() {
	_, err := exec.Command("youtube-dl", "--version").Output()
	if err != nil {
		log.Fatal("Youtube download (youtube-dl) isn't installed on the device")
	}
}

func (y *YtDownloader) Download(link string, updateEvent func(progress string)) error {
	baseDir := fmt.Sprintf("%s/%x",
		y.tempDirectory,
		sha256.Sum256([]byte(link)))
	err := os.MkdirAll(baseDir, os.ModePerm)
	err = os.MkdirAll(y.outputDirectory, os.ModePerm)
	if err != nil {
		y.logger.Error(err)
		return err
	}

	filePath := fmt.Sprintf("%s/%s", baseDir, "%(title)s-%(id)s.%(ext)s")

	command := exec.Command("youtube-dl",
		"-R 3",
		"-o",
		filePath,
		link,
	)

	var stdout io.ReadCloser
	stdout, err = command.StdoutPipe()

	go func() {
		for true {
			bytes := make([]byte, 1024)
			_, err = stdout.Read(bytes)
			if err != nil {
				return
			}
			output := string(bytes)

			compile := regexp.MustCompile(`[a-z\[\]\s]+([\d\.]+).[a-z\s]+([\d\.]+)([a-zA-Z]+)[a-z\s]+`)
			if err != nil {
				y.logger.Error(err)
				return
			}

			res := compile.FindAllStringSubmatch(output, -1)
			if len(res) != 0 && strings.Contains(res[0][0], "download") && len(res[0]) >= 2 {
				progress := res[0][1]

				y.logger.Debugw("progress",
					"percentage", progress,
					"link", link)
				updateEvent(progress)
			}

			time.Sleep(time.Millisecond * y.checkFrequencyMs)
		}
	}()

	err = command.Start()

	if err != nil {
		y.logger.Warn(err)
		return err
	}

	err = command.Wait()
	if err != nil {
		return err
	}

	var dir []fs.DirEntry
	dir, err = os.ReadDir(baseDir)
	if err != nil {
		y.logger.Error("Could not read directory")
		return err
	}

	for _, fileInfo := range dir {
		oldPath := fmt.Sprintf("%s/%s", baseDir, fileInfo.Name())
		newPath := fmt.Sprintf("%s/%s", y.outputDirectory, fileInfo.Name())

		if err := files.MoveFile(oldPath, newPath); err != nil {
			return err
		} else {
			y.logger.Infow("moved file",
				"fileName", fileInfo.Name())
			if err := os.Remove(baseDir); err != nil {
				y.logger.Warn("could not cleanup",
					"path", oldPath)
			}
		}
	}

	return nil
}
